/*
 *  ======== ti_ndk_config.c ========
 *  Configured module definitions for the NDK
 *
 *  DO NOT EDIT - This file is generated by the SysConfig tool.
 */


/*
 *  ======== ti.ndk/General ========
 */
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>

#include <signal.h>
#include <time.h>


#include <ti/drivers/emac/EMACMSP432E4.h>
#include <ti/ndk/inc/netmain.h>
#include <ti/ndk/inc/stkmain.h>
#include <ti/ndk/inc/os/oskern.h>

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>

/* forward declarations from other modules */
static void configTcp(void *hCfg);
static void configUdp(void *hCfg);
static void ti_ndk_config_ip_init(void *hCfg);

/* socket file descriptor table */
#define MAXSOCKETS 10
uint32_t ti_ndk_socket_max_fd = MAXSOCKETS;
void *ti_ndk_socket_fdtable[MAXSOCKETS];



/* NDK memory manager page size and number of pages [used by mmAlloc()] */
#define RAW_PAGE_SIZE 3072
#define RAW_PAGE_COUNT 6

const int ti_ndk_config_Global_rawPageSize  = RAW_PAGE_SIZE;
const int ti_ndk_config_Global_rawPageCount = RAW_PAGE_COUNT;

/* P.I.T. (page information table) */
#if defined(__ti__)
#pragma DATA_SECTION(ti_ndk_config_Global_pit, ".bss:NDK_MMBUFFER");
#pragma DATA_SECTION(ti_ndk_config_Global_pitBuffer, ".bss:NDK_MMBUFFER");
PITENTRY ti_ndk_config_Global_pit[RAW_PAGE_COUNT];
unsigned char ti_ndk_config_Global_pitBuffer[RAW_PAGE_SIZE * RAW_PAGE_COUNT];
#elif defined (__IAR_SYSTEMS_ICC__)
PITENTRY ti_ndk_config_Global_pit[RAW_PAGE_COUNT];
unsigned char ti_ndk_config_Global_pitBuffer[RAW_PAGE_SIZE * RAW_PAGE_COUNT];
#elif defined (__GNUC__)
PITENTRY ti_ndk_config_Global_pit[RAW_PAGE_COUNT] __attribute__ ((section(".bss:NDK_MMBUFFER")));
unsigned char ti_ndk_config_Global_pitBuffer[RAW_PAGE_SIZE * RAW_PAGE_COUNT] __attribute__ ((section(".bss:NDK_MMBUFFER")));
#else
PITENTRY ti_ndk_config_Global_pit[RAW_PAGE_COUNT];
unsigned char ti_ndk_config_Global_pitBuffer[RAW_PAGE_SIZE * RAW_PAGE_COUNT];
#endif

/* Memory bucket sizes */
#define SMALLEST 48
#define LARGEST (RAW_PAGE_SIZE)

const int ti_ndk_config_Global_smallest = SMALLEST;
const int ti_ndk_config_Global_largest  = LARGEST;

/* Memory Slot Tracking */
uint32_t ti_ndk_config_Global_Id2Size[] =
        {SMALLEST, 96, 128, 256, 512, 1536, LARGEST};

/*
 *  Local Packet Buffer Pool Definitions
 *
 *  The below variables/defines are used to override the defaults that are set
 *  in the Packet Buffer Manager (PBM) file src/stack/pbm/pbm_data.c
 */

/*
 *  Number of buffers in PBM packet buffer free pool
 *
 *  The number of buffers in the free pool can have a significant effect
 *  on performance, especially in UDP packet loss. Increasing this number
 *  will increase the size of the static packet pool use for both sending
 *  and receiving packets.
 */
#define PKT_NUM_FRAMEBUF 10

/* Size of Ethernet frame buffer */
#define PKT_SIZE_FRAMEBUF 1536

const int ti_ndk_config_Global_numFrameBuf = PKT_NUM_FRAMEBUF;
const int ti_ndk_config_Global_sizeFrameBuf = PKT_SIZE_FRAMEBUF;

/* Data space for packet buffers */
unsigned char ti_ndk_config_Global_pBufMem[PKT_NUM_FRAMEBUF * PKT_SIZE_FRAMEBUF] __attribute__ ((aligned(4), section(".bss:NDK_PACKETMEM")));
unsigned char ti_ndk_config_Global_pHdrMem[PKT_NUM_FRAMEBUF * sizeof(PBM_Pkt)] __attribute__ ((aligned(4), section(".bss:NDK_PACKETMEM")));

static char *hostname = "tisoc";

extern void llTimerTick();

/*
 *  ======== configTcp ========
 *  Configure the TCP settings
 *
 *  TCP configuration default values
 *
 *  Value   Configuration Property
 *  ----------------------------------------
 *  8192    TCP transmit buffer size
 *  8192    TCP receive buffer size (copy mode)
 *  8192    TCP receive size maximum (non-copy mode)
 *
 *  [1] Configuration entries are not added when the
 *      property is configured to its default value.
 *  [2] Default values are device specific.
 */
static void configTcp(void *hCfg)
{
    uint32_t tcpTxBufSize = 1024;
    uint32_t tcpRxBufSize = 1024;
    uint32_t tcpRxBufLimit = 2048;

    CfgAddEntry(hCfg, CFGTAG_IP, CFGITEM_IP_SOCKTCPTXBUF, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&tcpTxBufSize, NULL);
    CfgAddEntry(hCfg, CFGTAG_IP, CFGITEM_IP_SOCKTCPRXBUF, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&tcpRxBufSize, NULL);
    CfgAddEntry(hCfg, CFGTAG_IP, CFGITEM_IP_SOCKTCPRXLIMIT, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&tcpRxBufLimit, NULL);
}

/*
 *  ======== configUdp ========
 *  Configure the UDP settings
 */
static void configUdp(void *hCfg)
{
    uint32_t tcpRxBufSize = 2048;

    CfgAddEntry(hCfg, CFGTAG_IP, CFGITEM_IP_SOCKUDPRXLIMIT, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&tcpRxBufSize, NULL);
}

/*
 *  ======== ndkStackThread ========
 *  NDK main thread function
 */
static void ndkStackThread(uintptr_t arg0, uintptr_t arg1)
{
    extern void netIPAddrHook();
    void *hCfg;
    uint32_t u32cval;
    int rc;
    timer_t ndkHeartBeat;
    struct sigevent sev;
    struct itimerspec its;
    struct itimerspec oldIts;
    int ndkHeartBeatCount = 0;

    /* create the NDK timer tick */
    sev.sigev_notify = SIGEV_SIGNAL;
    sev.sigev_value.sival_ptr = &ndkHeartBeatCount;
    sev.sigev_notify_attributes = NULL;
    sev.sigev_notify_function = &llTimerTick;

    rc = timer_create(CLOCK_MONOTONIC, &sev, &ndkHeartBeat);
    if (rc != 0) {
        /* ndkStackThread: failed to create timer */
    }

    /* start the NDK 100ms timer */
    its.it_interval.tv_sec = 0;
    its.it_interval.tv_nsec = 100000000;
    its.it_value.tv_sec = 0;
    its.it_value.tv_nsec = 100000000;

    rc = timer_settime(ndkHeartBeat, 0, &its, NULL);
    if (rc != 0) {
        /* ndkStackThread: failed to set time */
    }

    /* user begin hook == null */

    rc = NC_SystemOpen(NC_PRIORITY_LOW, NC_OPMODE_INTERRUPT);
    if (rc) {
        /* ndkStackThread: NC_SystemOpen Failed */
        goto nc_fail;
    }

    /* create and build the system configuration from scratch */
    hCfg = CfgNew();
    if (!hCfg) {
        /* ndkStackThread: Unable to create configuration */
        goto main_exit;
    }

    /* user init hook == null */

    /* Add hostname */
    CfgAddEntry(hCfg, CFGTAG_SYSINFO, CFGITEM_DHCP_HOSTNAME, 0,
                strlen(hostname), (unsigned char *)hostname, NULL);

    /* TCP and UDP config */
    configTcp(hCfg);
    configUdp(hCfg);

    /* IP configuration */
    ti_ndk_config_ip_init(hCfg);

    /* config low priority task stack size */
    u32cval = 2048;
    CfgAddEntry(hCfg, CFGTAG_OS, CFGITEM_OS_TASKSTKLOW, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&u32cval, NULL);

    /* config normal priority task stack size */
    u32cval = 2048;
    CfgAddEntry(hCfg, CFGTAG_OS, CFGITEM_OS_TASKSTKNORM, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&u32cval, NULL);

    /* config high priority task stack size */
    u32cval = 2048;
    CfgAddEntry(hCfg, CFGTAG_OS, CFGITEM_OS_TASKSTKHIGH, CFG_ADDMODE_UNIQUE,
            sizeof(uint32_t), (unsigned char *)&u32cval, NULL);

    /*  Boot the system using this configuration
     *
     *  Loop until the function returns 0. This facilitates a reboot command.
     */
    do
    {
        rc = NC_NetStart(hCfg, NULL, NULL, netIPAddrHook);

        /* user reboot hook == null */

    } while (rc > 0);

    /* user delete hook == null */

    /* shut down the stack */
    CfgFree(hCfg);

main_exit:
    NC_SystemClose();

nc_fail:
    /* stop and delete the NDK heartbeat */
    its.it_value.tv_sec = 0;
    its.it_value.tv_nsec = 0;
    rc = timer_settime(ndkHeartBeat, 0, &its, &oldIts);
    rc = timer_delete(ndkHeartBeat);

    /* ndkStackThread: exiting ... */
}

/*
 * ======== ti_ndk_config_Global_startupFxn ========
 * Called to start up the NDK. In BIOS, this can be called as a BIOS startup
 * function, or from main(). In FreeRTOS, this should be called from main().
 */
void ti_ndk_config_Global_startupFxn()
{
    Task_Params params;
    Task_Handle ndkThread;

    Task_Params_init(&params);
    params.instance->name = "ndkStackThread";
    params.priority = 5;
    params.stackSize = 2048;

    ndkThread = Task_create((Task_FuncPtr)ndkStackThread, &params, NULL);

    if (!ndkThread) {
        /* Error: could not create NDK stack thread */
        while(1);
    }
}


/*
 *  ======== ti.ndk/NIMU ========
 */
NIMU_DEVICE_TABLE_ENTRY NIMUDeviceTable[2] = {
    {
        .init = EMACMSP432E4_NIMUInit
    },
    {NULL}
};

/*
 *  ======== ti_ndk_config_ip_init ========
 *  Configure the IP settings
 */
static void ti_ndk_config_ip_init(void *hCfg)
{
    CI_SERVICE_DHCPC dhcpc;
    static unsigned char DHCP_OPTIONS[] = { DHCPOPT_SUBNET_MASK };

    /* ----------------- Configuring IP for CONFIG_NIMU_0 ----------------- */
    /* NOTE!  The application must provide the service report function
     * matching this prototype!
     */
    extern void serviceReport(uint32_t item, uint32_t status,
            uint32_t report, void *h);


    /* use DHCP to obtain IP address on interface  */
    memset(&dhcpc, 0, sizeof(dhcpc));
    /*
     * CIS_FLG_IFIDXVALID makes this DHCP Client bind to the ip address of this
     * NIMU interface, 1. This is the only appropriate mode for the DHCP
     * Client.
     */
    dhcpc.cisargs.Mode = CIS_FLG_IFIDXVALID;
    dhcpc.cisargs.IfIdx = 1;
    dhcpc.cisargs.pCbSrv = serviceReport;
    dhcpc.param.pOptions = DHCP_OPTIONS;
    dhcpc.param.len = 1;
    CfgAddEntry(hCfg, CFGTAG_SERVICE, CFGITEM_SERVICE_DHCPCLIENT, 0,
            sizeof(dhcpc), (unsigned char *)&dhcpc, NULL);

}

